import 'package:flutter/material.dart';

import 'circle.dart';

class BlinkingPoint extends StatefulWidget {
  final double xCoor;
  final double yCoor;
  final Color pointColor;
  final double pointSize;

  BlinkingPoint({
    required this.xCoor,
    required this.yCoor,
    required this.pointColor,
    required this.pointSize,
  });

  @override
  BlinkingPointState createState() => new BlinkingPointState();
}

class BlinkingPointState extends State<BlinkingPoint>
    with SingleTickerProviderStateMixin {
  late Animation animation;
  late AnimationController animationController;

  @override
  Widget build(BuildContext context) {
    return Align(
      child: Center(
        child: LogoAnimation(
          xCoor: widget.xCoor,
          yCoor: widget.yCoor,
          pointColor: widget.pointColor,
          pointSize: widget.pointSize,
          animation: animation,
        ),
      ),
    );
  }

  @override
  void dispose() {
    animationController.dispose();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    animationController = AnimationController(
      duration: Duration(milliseconds: 1000),
      vsync: this,
    );

    animation =
        Tween(begin: 0.0, end: widget.pointSize).animate(animationController);
    animation.addStatusListener((status) {
      if (status == AnimationStatus.completed) {
        animationController.reset();
      } else if (status == AnimationStatus.dismissed) {
        animationController.forward();
      }
    });
    animationController.forward();
  }
}

class LogoAnimation extends AnimatedWidget {
  final double xCoor;
  final double yCoor;
  final Color pointColor;
  final double pointSize;

  LogoAnimation({
    required Animation animation,
    required this.xCoor,
    required this.yCoor,
    required this.pointColor,
    required this.pointSize,
  }) : super(listenable: animation);

  @override
  Widget build(BuildContext context) {
    Animation? animation = listenable as Animation?;
    return new CustomPaint(
      foregroundPainter: Circle(
        xCoordinate: xCoor,
        yCoordinate: yCoor,
        color: pointColor,
        pointSize: pointSize,
        blinkRadius: animation!.value,
      ),
    );
  }
}
